# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-06-25 04:38
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('page', '0002_page_version'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='page_version',
            options={'ordering': ['-saved']},
        ),
        migrations.AlterField(
            model_name='page',
            name='lastmod',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name='page',
            name='sitemap',
            field=models.BooleanField(default=True, verbose_name='Share a link to this page with search engines'),
        ),
        migrations.AlterField(
            model_name='page',
            name='title',
            field=models.CharField(max_length=100, verbose_name='Menu Title'),
        ),
        migrations.AlterField(
            model_name='page_version',
            name='saved',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]



import os
import datetime
import re

from django.db import models
from django.conf import settings
from django.core.cache import cache
import hashlib
from django.contrib.auth.models import User
from django.apps import apps
from django.utils import timezone
from django.core.exceptions import ObjectDoesNotExist


re_alphaplus = re.compile("[^a-zA-Z0-9/\.]")
re_page_slug = re.compile('^[\w\-_/]+$')

class Invalid_Page_Name(Exception):
    pass

PAGE_SITE = getattr(settings, 'PAGE_SITE', 'page.Site')
PAGE_PAGE = getattr(settings, 'PAGE_PAGE', 'page.Page')
PAGE_USER = getattr(settings, 'PAGE_USER', 'page.Page_User')
PAGE_CACHE_EXPIRE =getattr(settings, 'PAGE_CACHE_EXPIRE', 60*60*4 ) # four hour
PAGE_PAGE_CACHE= getattr(settings, 'PAGE_PAGE_CACHE', False) # disable page_cache by default
PAGE_SITE_CACHE= getattr(settings, 'PAGE_SITE_CACHE', False) # disable page_cache by default



def get_class_model_perm(class_model,perm):
    (model_class,model) = get_class_model(PAGE_PAGE)
    perm = "%s.%s_%s" %(model_class,perm,model)
    return perm.lower()
 
def get_class_model(class_model):
    return class_model.split('.')

#monkey patch user to show fullname  Subclassing breaks manytomany deletes
def user_unicode(self):
    return self.get_full_name()

User.__unicode__ = user_unicode 

#site
def get_page_site_key(domain_name):
    return "page_site/%s" % domain_name

def get_site_model():
    return apps.get_model ( *PAGE_SITE.split('.',1) )

def get_page_site(request):
    page_site = None
    if PAGE_SITE_CACHE:#try cache
        key = get_page_site_key(request.get_host())
        page_site = cache.get(key,None)

    if page_site:
        return page_site
    else:
        page_site_model = get_site_model()
        page_site = page_site_model.get_site(request)
        if PAGE_SITE_CACHE:
            cache.set(key,page_site,PAGE_CACHE_EXPIRE)
        return page_site

def get_page_site_version_key(domain_name):
    return "page_site_version/%s" % domain_name
   
def get_page_site_version(request):
    if PAGE_PAGE_CACHE:
        key = get_page_site_version_key(request.get_host())
        return cache.get(key,0)
    else:
        return 0

def invalidate_site_cache(request):
    if PAGE_PAGE_CACHE:
        domain_name = request.get_host()
        try:
            cache.incr(get_page_site_version_key(domain_name))
        except ValueError:
            cache.set(get_page_site_version_key(domain_name),1)

        cache.delete(get_page_site_key(domain_name))
        if (request.page_site):
            cache.delete(get_page_site_key(request.page_site.domain_name))

def get_page_site_user():
    page_user =  apps.get_model ( *PAGE_USER.split('.',1) )
    return page_user

#page
def get_page_model():
    return apps.get_model ( *PAGE_PAGE.split('.',1) )

def get_page(request):
    slug = request.path[1:].lower().rstrip('/')
    if slug == "":
        slug="default"

    page = None

    if PAGE_PAGE_CACHE:
        key = get_page_key(request.page_site.pk,slug)

        if not request.GET.get('refresh') :
            page = cache.get(key,None)

    if not page:
        try:
            #print "looking for page slug %s" % slug
            page_model =  get_page_model()

            page = page_model.objects.get(slug=slug,site=request.page_site)
            #print("looking for page page %s" % slug)
            if PAGE_PAGE_CACHE:
                cache.set(key,page,PAGE_CACHE_EXPIRE)
        except ObjectDoesNotExist:
            page = None 
                
    return page


def get_page_key(site_pk,slug):
    #print "get_page slug='%s'" % slug

    if re_page_slug.match(slug) is None:
        raise Invalid_Page_Name

    #use cache if there
    key = "%s/page/%s" % (site_pk,slug)
    #key = hashlib.md5(key).hexdigest()
    return key

    
class Site_Abstract(models.Model):
    class Meta:
        abstract = True
        
    title = models.CharField(max_length = 200)
    domain_name = models.CharField(max_length = 300,primary_key=True)
    default_domain = models.BooleanField(default=False)

    meta_data = models.TextField(blank=True,null=True)
    footer = models.TextField(blank=True,null=True)

    def __str__(self):
        return self.title

    @staticmethod
    def get_site(request):
        try:
            page_site = get_site_model().objects.get(domain_name = request.get_host())
        except ObjectDoesNotExist:
            try:
                page_site = get_site_model().objects.filter(default_domain = True)[0]
            except IndexError: #create the site
                page_site = get_site_model()(domain_name = request.get_host(),title=request.get_host())
                page_site.save()
        return page_site


if PAGE_SITE == 'page.Site':
    class Site(Site_Abstract):
        pass     
    
class Page_User(User):
    class Meta:
        proxy = True
        
    @staticmethod
    def get_site_users(site):
        """overide this model and supply this function to use custom users, for example, based on site"""
        return Page_User.objects.all()



class Page_Abstract(models.Model):
    class Meta:
        abstract = True
        permissions = (
            ("change_show_in_menu", "change_show_in_menu"),
        )

    site = models.ForeignKey(PAGE_SITE)
    slug = models.CharField(max_length = 100,db_index=True)
    title = models.CharField(max_length = 100,verbose_name="Menu Title")
    show_in_menu = models.BooleanField(default=False)
    priority = models.IntegerField(default=0)
    html_title = models.CharField(max_length = 100,null=True,blank=True)
    content = models.TextField()
    members_only = models.BooleanField(default=False)
    lastmod = models.DateTimeField(default=timezone.now)
    sitemap  = models.BooleanField(default=True,verbose_name="Share a link to this page with search engines")


    def get_absolute_url(self):
        return "/%s" % self.slug

    def __str__(self):
        return self.slug

class Page_Editor(models.Model):
    page = models.ForeignKey(PAGE_PAGE,db_index=True)
    editor = models.ForeignKey(PAGE_USER,db_index=True)
    
    def __str__(self):
        return "%s %s" % (self.page,self.editor)

if PAGE_PAGE == 'page.Page':
    class Page(Page_Abstract):
        pass

class Page_Version(models.Model):
    page = models.ForeignKey(PAGE_PAGE)
    saved = models.DateTimeField(default=timezone.now)
    data = models.TextField()
    class Meta:
        ordering = ["-saved"]


    def __str__(self):
        return "%s:%s %s" % (self.page.site,self.page.slug,self.saved)

def upload_uuid_file_to(instance,filename):
    path = 'page/%s' % instance.uuid
    return get_filename(path,filename,255)

class UUID_File(models.Model):
    file = models.FileField(max_length=255,upload_to=upload_uuid_file_to)
    uuid = models.CharField(max_length=100)

    def filename(self):
        return os.path.basename(self.file.name)
    
    def __str__(self):
        return self.file.name

def get_filename(path,filename,max_len):
    dst = '%s/%s' % (path,filename)
    if len(dst) > max_len:
        dst = '%s/%s' % (path,hashlib.md5(filename).hexdigest())

    return dst


def upload_file_to(instance,filename):
#    ext = os.path.splitext(filename)[1]
#    ext = ext.lower()
    path = 'page/%s' % instance.page.pk
    return get_filename(path,filename,255)

class File(models.Model):
    file = models.FileField(max_length=255,upload_to=upload_file_to)
    page = models.ForeignKey(PAGE_PAGE,related_name = 'page_file')


    def filename(self):
        return os.path.basename(self.file.name)

    def __str__(self):
        return self.file.name





# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-08-08 03:07
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('page', '0004_auto_20170807_2207'),
    ]

    operations = [
        migrations.AlterField(
            model_name='page',
            name='content',
            field=models.TextField(blank=True, null=True),
        ),
    ]



from django.conf import settings
from django.dispatch import receiver
import os
import datetime
from django.core.cache import cache
from django.db.models.signals import pre_save,pre_delete,post_save
from .models import Site_Abstract, Page_Version, File, Page_Abstract,get_page_site_key
from django.core import serializers
from django.apps import apps
get_model = apps.get_model
from django.utils import timezone

PAGE_PAGE = getattr(settings, 'PAGE_PAGE', 'page.Page')
PAGE_PAGE_CACHE= getattr(settings, 'PAGE_PAGE_CACHE', False) # disable page_cache by default
PAGE_SITE_CACHE= getattr(settings, 'PAGE_SITE_CACHE', False) # disable page_cache by default

USE_BEAUTIFULSOUP = getattr(settings, 'PAGE_SITE_USE_BEAUTIFULSOUP', True)
if USE_BEAUTIFULSOUP:
    from bs4 import BeautifulSoup


@receiver(pre_save, sender=Site_Abstract,dispatch_uid='page_site_pre_save')
def pre_save_site_callback(instance, sender, **kwargs):
    if PAGE_SITE_CACHE:
        cache.delete(get_page_site_key(instance.domain_name))

@receiver(pre_delete, sender=Site_Abstract,dispatch_uid='page_site_pre_delete')
def pre_delete_site_callback(instance, sender, **kwargs):
    if PAGE_SITE_CACHE:
        cache.delete(get_page_site_key(instance.domain_name))

@receiver(post_save,sender=get_model(PAGE_PAGE))
def post_save_page_callback(instance, sender, **kwargs):
    if not issubclass(sender, Page_Abstract):
       return
    version = Page_Version(page=instance)
    version.data = serializers.serialize("json", [instance,])
    version.save()

#@receiver(pre_save, sender=Page_Abstract,dispatch_uid='page_page_pre_save')
@receiver(pre_save,sender=get_model(PAGE_PAGE))
def pre_save_page_callback(instance, sender, **kwargs):
    if not issubclass(sender, Page_Abstract):
       return

    if USE_BEAUTIFULSOUP:
        tree = BeautifulSoup(instance.content, "html.parser")
        instance.content = tree.prettify()
    instance.lastmod = timezone.now()

@receiver(pre_delete, sender=File,dispatch_uid='page_file_pre_delete')
def pre_delete_file_callback(instance, sender, **kwargs):
    try:
        os.remove(instance.file.path)
    except: pass



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-08-08 03:07
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('page', '0003_auto_20170624_2338'),
    ]

    operations = [
        migrations.AlterField(
            model_name='page',
            name='site',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='churchnest_site.Churchnest_Site'),
        ),
        migrations.DeleteModel(
            name='Site',
        ),
    ]



from uuid import uuid4
from django.core.files import File
from django.core.files.temp import NamedTemporaryFile
import os
from django.views.decorators.csrf import csrf_exempt, csrf_protect
from django.conf import settings
import json

from django.http import HttpResponseRedirect
from django.contrib.auth.views import login
from django.contrib.auth.decorators import login_required
from django.views.generic import TemplateView
import json
from django.http import HttpResponse
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit,Field,Layout,ButtonHolder,Fieldset,HTML

from django.utils.decorators import method_decorator
from django.http import HttpResponse
from django.core.urlresolvers import reverse 
from django.views.generic import View
from django.views.generic import CreateView,ListView,UpdateView,DeleteView


from . import forms
from . import models

class Edit(UpdateView):
    model = models.get_site_model()
    form_class = forms.Site
    #template_name = 'news.html'
    
    def get_initial(self):
        initial = super(Edit, self).get_initial()
        if not self.object:
            initial['default_domain'] = True
        return initial

    def form_valid(self, form):
        models.invalidate_site_cache(self.request)
        return super(Edit, self).form_valid(form)
    
    def get_object(self, queryset=None):
        return self.request.page_site

    def get_success_url(self):
        if self.request.POST.get('submit') == 'Save':
            return reverse("site_edit") 
        return "/%s" % self.request.GET.get('next','')




from . import models
from django.contrib import admin
from django.conf import settings

PAGE_SITE = getattr(settings, 'PAGE_SITE', 'page.Site')
PAGE_PAGE = getattr(settings, 'PAGE_PAGE', 'page.Page')
PAGE_USER = getattr(settings, 'PAGE_USER', 'page.Page_User')


if PAGE_PAGE == 'page.Page':
    admin.site.register(models.Page)
admin.site.register(models.Page_Version)

admin.site.register(models.File)
admin.site.register(models.UUID_File)

if PAGE_SITE == 'page.Site':
    admin.site.register(models.Site)
admin.site.register(models.Page_Editor)
#test


from django.http import HttpResponseRedirect
from django.contrib.auth.decorators import login_required
from django.views.generic import TemplateView
from django.http import HttpResponse


from django.utils.decorators import method_decorator
from django.core.urlresolvers import reverse 

from . import models
from django.shortcuts import get_object_or_404

class Editor_View:

    def get_page(self):
        page = get_object_or_404(models.get_page_model(),site=self.request.page_site,slug=self.kwargs['slug'])
        return page

class Delete(TemplateView,Editor_View):
    template_name = 'page/editor_confirm_delete.html'

    def get_context_data(self, **kwargs):
        page = self.get_page()
        editor = get_object_or_404(models.Page_Editor,pk=self.kwargs['editor'])

        return {
            'editor': editor,
            'page': page 
        }


    def post(self, request, *args, **kwargs):
        page = self.get_page()
        page_editor = get_object_or_404(models.Page_Editor,pk=self.kwargs['editor'])
        if page_editor and (page_editor.page == page):
            page_editor.delete()

            if self.request.is_ajax():
                return HttpResponse('Success')
        return HttpResponseRedirect(reverse("page_edit",args=[page.slug])) 

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        if not request.user.has_perm('page.delete_page_editor') :
            return HttpResponse('Sorry, You do not have rights!')
        
        return super(Delete, self).dispatch(request,*args, **kwargs)
    
    
class Add(TemplateView,Editor_View):
    template_name = 'page/editor_form.html'

    found_editors = None
    
    def get_context_data(self, **kwargs):

        return {
            'found_editors': self.found_editors 
        }

     
    def post(self, request, *args, **kwargs):
        add_editor = self.request.POST.get('add_editor')
        
        if add_editor:
            user = get_object_or_404(models.get_page_site_user(),pk=add_editor)
            page = self.get_page()
            page_editor = models.Page_Editor(editor=user,page=page)
            page_editor.save()
            if self.request.is_ajax():
                return HttpResponse('Success')
            else:
                return HttpResponseRedirect(reverse("page_edit",args=[page.slug])) 
            
        else:
            username = self.request.POST.get('username')
            first_name = self.request.POST.get('first_name')
            last_name = self.request.POST.get('last_name')
            
            q = models.get_page_site_user().get_site_users(self.request.page_site)
            if username:
                q = q.filter(username__iexact=username)
    
            if first_name:
                q = q.filter(first_name__iexact=first_name)
    
            if last_name:
                q = q.filter(last_name__iexact=last_name)
            
            self.found_editors = q
            
            return self.get(request,*args, **kwargs)

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        if not request.user.has_perm('page.add_page_editor') :
            return HttpResponse('Sorry, You do not have rights!')
        else:
            return super(Add, self).dispatch(request,*args, **kwargs)



import re
import feedparser
from django.core.cache import cache
from django.template import RequestContext
from django.template.loader import render_to_string

re_rss = re.compile('display="rss"')
re_feed = re.compile('feed="([^"]*)"')
re_max_results = re.compile('max_results="([^"]*)"')
re_headers_only = re.compile('headers_only')

import hashlib

cache_expire = 60 * 60 * 1  # one hour


def get(comment, request):
    """Takes comment and replaces any news tags with the news"""
    # news_tags = soup.findAll(name="news")
    max_results = 3

    if re_rss.search(comment):
        key = hashlib.md5(comment.encode('utf-8')).hexdigest()
        if not request.GET.get('refresh'):
            # try to get cache
            result = cache.get(key, None)
            if result:
                return result

        # re_max_results
        max_results_re_result = re_max_results.search(comment)
        if max_results_re_result:
            max_results = int(max_results_re_result.group(1))

        # include
        feed_re = re_feed.search(comment)
        if feed_re:
            template_values = {}
            template_values['feed'] = feedparser.parse(feed_re.group(1)).entries[:max_results]

            if re_headers_only.search(comment):
                template_values['headers_only'] = False
            else:
                template_values['headers_only'] = False

            response = render_to_string('page/rss.html', template_values)
            cache.set(key, response, cache_expire)
            return response



from django.conf.urls import  url
from . import views_page,views_site,views_editor


urlpatterns = [
     url(r'site/edit/$', views_site.Edit.as_view(),name='site_edit' ),

     url(r'add/upload/$', views_page.Upload_File.as_view(),name='page_add_upload' ),
     url(r'add/links/$', views_page.Edit_Links.as_view(),name='page_add_links' ),
     url(r'add/$', views_page.Add.as_view(),name='page_add' ),
     url(r'list/$', views_page.List.as_view(),name='page_list' ),
     url(r'(?P<slug>.*)/edit/links/$', views_page.Edit_Links.as_view(),name='page_edit_links' ),
     url(r'(?P<slug>.*)/edit/images/$', views_page.Edit_Images.as_view(),name='page_edit_images' ),
     url(r'(?P<slug>.*)/edit/editor/add$', views_editor.Add.as_view(),name='editor_add' ),
     url(r'(?P<slug>.*)/edit/editor/delete/(?P<editor>.*)/$', views_editor.Delete.as_view(),name='editor_delete' ),
     url(r'(?P<slug>.*)/edit/$', views_page.Edit.as_view(),name='page_edit' ),
     url(r'(?P<slug>.*)/delete/$', views_page.Delete.as_view(),name='page_delete' ),

]




#stolen from django
from django.core import exceptions
from django.utils.importlib import import_module

def get_class(path):
    try:
        mw_module, mw_classname = path.rsplit('.', 1)
    except ValueError:
        raise exceptions.ImproperlyConfigured('%s isn\'t a module' % path)
    try:
        mod = import_module(mw_module)
    except ImportError as e:
        raise exceptions.ImproperlyConfigured('Error importing  %s: "%s"' % (mw_module, e))
    try:
        mw_class = getattr(mod, mw_classname)
        return mw_class
    
    except AttributeError:
        raise exceptions.ImproperlyConfigured('Module "%s" does not define a "%s" class' % (mw_module, mw_classname))



# -*- coding: utf-8 -*-


from django.db import models, migrations
import datetime


class Migration(migrations.Migration):

    dependencies = [
        ('page', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Page_Version',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('saved', models.DateTimeField(default=datetime.datetime.now)),
                ('data', models.TextField()),
                ('page', models.ForeignKey(to='page.Page')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



from django.contrib import sitemaps
from django.core.urlresolvers import reverse
from . import models
from django.contrib.sitemaps import Sitemap
import datetime

class PageSitemap(Sitemap):
    changefreq = "monthly"
    priority = 0.6
    protocol = 'http'


    def items(self):
        return models.get_page_model().objects.filter(sitemap=True,members_only=False)

    def lastmod(self, obj):
        return obj.lastmod



import importlib
import sys, os
from django.views.decorators.csrf import csrf_protect
import json
from django.template.response import TemplateResponse
from django.core.urlresolvers import reverse
from django.conf import settings
from django.utils.functional import SimpleLazyObject

from django.http import Http404
from django.http import HttpResponseRedirect

from . import models
import re

#add path for importation of plugins
sys.path.append(os.path.join(os.path.realpath(os.path.dirname(__file__)), '..'))



PAGE_PLUGINS = getattr(settings, 'PAGE_PLUGINS', ())

reg_v = re.compile(
    r"1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\\-(n|u)|c55\\/|capi|ccwa|cdm\\-|cell|chtm|cldc|cmd\\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\\-s|devi|dica|dmob|do(c|p)o|ds(12|\\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\\-|_)|g1 u|g560|gene|gf\\-5|g\\-mo|go(\\.w|od)|gr(ad|un)|haie|hcit|hd\\-(m|p|t)|hei\\-|hi(pt|ta)|hp( i|ip)|hs\\-c|ht(c(\\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\\-(20|go|ma)|i230|iac( |\\-|\\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\\/)|klon|kpt |kwc\\-|kyo(c|k)|le(no|xi)|lg( g|\\/(k|l|u)|50|54|e\\-|e\\/|\\-[a-w])|libw|lynx|m1\\-w|m3ga|m50\\/|ma(te|ui|xo)|mc(01|21|ca)|m\\-cr|me(di|rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\\-2|po(ck|rt|se)|prox|psio|pt\\-g|qa\\-a|qc(07|12|21|32|60|\\-[2-7]|i\\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\\-|oo|p\\-)|sdk\\/|se(c(\\-|0|1)|47|mc|nd|ri)|sgh\\-|shar|sie(\\-|m)|sk\\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\\-|v\\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\\-|tdg\\-|tel(i|m)|tim\\-|t\\-mo|to(pl|sh)|ts(70|m\\-|m3|m5)|tx\\-9|up(\\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|xda(\\-|2|g)|yas\\-|your|zeto|zte\\-",
    re.I | re.M)
re_comment = re.compile(r"<!--(.*?)-->")


class page_fallback_middleware(object):
    def process_request(self, request):
        request.page_site = models.get_page_site(request)
        if models.PAGE_USER == 'page.Page_User':
            #TODO fix this double lookup!
            request.page_user = SimpleLazyObject(lambda: models.Page_User.objects.get(pk=request.user.pk))
        else:
            request.page_user = SimpleLazyObject(
                lambda: models.get_page_site_user().objects.filter(user=request.user)[0])
        return None

    def process_response(self, request, response):
        request.page = None

        if response.status_code != 404:
            return response # No need to check for a flatpage for non-404 responses.
        try:
            return page_view(request)


        # Return the original response if any errors happened. Because this
        # is a middleware, we can't assume the errors will be caught elsewhere.
        except Http404:
            return response

#        except AttributeError:
#            return response
        except:
            if settings.DEBUG:
                raise
            return response

@csrf_protect
def page_view(request):
    context = {}

    page = models.get_page(request)


    if request.GET.get('history'):
        history = page.page_version_set.all()
        version_id = request.GET.get('version')
        if version_id:
            for version in history:
                if version.pk == int(version_id):
                    context['version'] = version
                    obj = json.loads(version.data)[0]['fields']
                    for k, v in obj.items():
                        excludes = ["post_ptr", 'site']
                        if not k in excludes:
                            setattr(page, k, v)
                    if request.GET.get('revert') and request.user.has_perm(models.get_class_model_perm(models.PAGE_PAGE, 'change')):
                        page.save()
                        history = None
                        break
        context['history'] = history

    if page and page.content:
        if page and page.members_only and not request.user.is_authenticated():
            return HttpResponseRedirect(reverse('login') + "?next=/" + page.slug)

        request.page = page
        #print "page banners = %s" % page.banners

        page.new_content = '' #recreate response, since wecan't modofy page.content and use search
        new_content_pos = 0

        #re
        if True:
            matches = re_comment.finditer(page.content)
            for match in matches:
                #import logging
                #logging.debug("match:%s" % match.group(0))
                comment = match.group(0)
                for plugin in PAGE_PLUGINS:
                    try:
                        mod = importlib.import_module(plugin)
                    except ImportError as e:
                        raise ImportError(
                            "Could not import settings '%s' (Is it on sys.path? Does it have syntax errors?)" % plugin)

                    response = mod.get(comment, request)
                    if response:
                        page.new_content += page.content[new_content_pos:match.start()] + response
                        new_content_pos = match.end()
            page.new_content += page.content[new_content_pos:]
            page.content = page.new_content
    context['slug'] = request.path[1:]
    context['object'] = page
    context['page'] = page
    context['user_page_add'] = request.user.has_perm(models.get_class_model_perm(models.PAGE_PAGE, 'add'))

    #print 'set context'
    if page:
        if request.user.has_perm(models.get_class_model_perm(models.PAGE_PAGE, 'change')):
            context['user_page_edit'] = True
        elif request.user.is_authenticated() and models.Page_Editor.objects.filter(editor=request.page_user,
                                                                                   page=page).count():
            context['user_page_edit'] = True
        else:
            context['user_page_edit'] = False

    template = 'page/page_detail.html'

    return TemplateResponse(
        request=request,
        template=template,
        context=context
    ).render()


from uuid import uuid4
import json

from django.core.files import File
from django.views.decorators.csrf import csrf_exempt
from django.conf import settings
from django.http import HttpResponseRedirect, HttpResponseNotFound
from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator
from django.http import HttpResponse
from django.core.urlresolvers import reverse
from django.views.generic import View
from django.views.generic import CreateView, ListView, UpdateView, DeleteView
from django.core.cache import cache
from django.views.generic.edit import FormMixin
from django.core.files.uploadedfile import UploadedFile
from . import forms
from . import models


class List(ListView):
    model = models.get_page_model()
    context_object_name = "page_list"  # needed for plugins
    template_name = 'page/page_list.html'

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(List, self).dispatch(request, *args, **kwargs)


class Page_Form_Base(FormMixin):
    def get_form_kwargs(self):
        kwargs = super(Page_Form_Base, self).get_form_kwargs()
        kwargs['site'] = self.request.page_site
        return kwargs

    def get_success_url(self):
        if self.request.POST.get('submit') == 'Save':
            return reverse("page_edit", args=(self.object.slug,))
        else:
            return "/%s" % self.object.slug

    def get_context_data(self, **kwargs):
        context = super(Page_Form_Base, self).get_context_data(**kwargs)
        context['uuid'] = uuid4()
        context['PAGE_TINYMCE_PLUGINS'] = getattr(settings, 'PAGE_TINYMCE_PLUGINS', "")
        context['PAGE_TINYMCE_BUTTONS'] = getattr(settings, 'PAGE_TINYMCE_BUTTONS', "")
        return context

    def get_form_class(self):
        if self.request.user.has_perm('page.change_show_in_menu'):
            return forms.Page_Show_In_Menu
        else:
            return forms.Page

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.site = self.request.page_site
        if not self.object.pk:
            page_editor = models.Page_Editor(editor=self.request.page_user)
        else:
            page_editor = None

        self.object.save()
        form.save_m2m()

        if page_editor:
            page_editor.page = self.object
            page_editor.save()

        # save files
        for attachment in self.request.FILES.getlist('file'):
            if attachment:
                report_attachment = models.File(page=self.object, file=attachment)
                report_attachment.save()

                # save html5
        for attachment in models.UUID_File.objects.filter(uuid=self.request.POST.get('uuid')):
            if not str(attachment.pk) in self.request.POST.getlist('remove_new_file'):
                page_file = models.File(page=self.object, file=attachment.file)
                page_file.save()
                attachment.delete()

        if len(self.request.FILES.getlist('file')):
            self.object.save()

            # remove files
        for file_pk in self.request.POST.getlist('remove_file'):
            try:
                file_remove = models.File.objects.get(pk=file_pk, )
                file_remove.delete()
            except models.File.DoesNotExist:
                pass

        cache.delete(models.get_page_key(self.request.page_site.pk, self.object.slug))
        return HttpResponseRedirect(self.get_success_url())


class Add(Page_Form_Base, CreateView):
    model = models.get_page_model()
    form_class = forms.Page
    template_name = 'page/page_form.html'

    def get_initial(self):
        initial = super(Add, self).get_initial()
        initial['slug'] = self.request.GET.get('slug', '').rstrip('/')
        if initial['slug'] == "":
            initial['slug'] = "default"

        initial['title'] = initial['slug']

        return initial

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        if not request.user.has_perm(models.get_class_model_perm(models.PAGE_PAGE, 'add')):
            return HttpResponse('Sorry, You do not have access!')
        return super(Add, self).dispatch(request, *args, **kwargs)


class Edit(Page_Form_Base, UpdateView):
    model = models.get_page_model()
    template_name = 'page/page_form.html'

    def get_object(self, queryset=None):
        try:
            page = models.get_page_model().objects.get(site=self.request.page_site, slug=self.kwargs['slug'])
        except:
            page = None
        return page

    def get_context_data(self, **kwargs):
        kwargs = super(Edit, self).get_context_data(**kwargs)
        kwargs['editors'] = models.Page_Editor.objects.filter(page=self.object)
        kwargs['files'] = models.File.objects.filter(page=self.object)
        return kwargs

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        if not self.object:
            return HttpResponseNotFound('Page not found')

        if not self.request.user.has_perm(models.get_class_model_perm(models.PAGE_PAGE, 'change')):
            if models.Page_Editor.objects.filter(editor=request.page_user, page=self.object).count() < 1:
                return HttpResponse('Sorry, You do not have access!')

        return super(Edit, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        if not self.request.user.has_perm(models.get_class_model_perm(models.PAGE_PAGE, 'change')):
            if models.Page_Editor.objects.filter(editor=request.page_user, page=self.object).count() < 1:
                return HttpResponse('Sorry, You do not have access!')

        return super(Edit, self).post(request, *args, **kwargs)

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(Edit, self).dispatch(request, *args, **kwargs)


class Delete(DeleteView):
    model = models.get_page_model()
    template_name = 'page/page_confirm_delete.html'

    def get_success_url(self):
        cache.delete(models.get_page_key(self.request.page_site.pk, self.object.slug))
        return reverse("page_list")

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        if not request.user.has_perm(models.get_class_model_perm(models.PAGE_PAGE, 'delete')):
            return HttpResponse('Sorry, You do not have rights!')

        return super(Delete, self).dispatch(request, *args, **kwargs)


class Edit_Links(View):
    def get(self, request, *args, **kwargs):
        response = "["
        wrote_first = False

        response += "{\"title\": \"Files\", \"menu\": ["

        uuid = self.request.GET.get('uuid')
        if uuid:
            for attachment in models.UUID_File.objects.filter(uuid=uuid):
                if (wrote_first):
                    response += ","
                else:
                    wrote_first = True
                response += "{\"title\": \"%s\", value: \"%s\"}" % (attachment.filename(), attachment.file.url)

        if 'slug' in self.kwargs:
            page = models.get_page_model().objects.get(site=self.request.page_site, slug=self.kwargs['slug'])
            if page:
                for file in page.page_file.all():
                    if (wrote_first):
                        response += ","
                    else:
                        wrote_first = True
                    response += "{\"title\": \"%s\", \"value\": \"%s\"}" % (file.filename(), file.file.url)
        response += "]},"
        wrote_first = False

        response += "{\"title\": \"Pages\", \"menu\": ["

        # get all pages
        pages = models.get_page_model().objects.all().filter(site=self.request.page_site).order_by('slug')
        for page_entry in pages:
            if (wrote_first):
                response += ","
            else:
                wrote_first = True
            response += "{\"title\": \"%s\", \"value\": \"%s\"}" % (page_entry.slug, page_entry.slug)
        response += "]}"

        response += "]"
        return HttpResponse(response)


class Edit_Images(View):
    def get(self, request, *args, **kwargs):
        response = "["
        wrote_first = False

        uuid = self.request.GET.get('uuid')
        if uuid:
            for attachment in models.UUID_File.objects.filter(uuid=uuid):
                if (wrote_first):
                    response += ","
                else:
                    wrote_first = True
                response += "{title: '%s', value: '%s'}" % (attachment.filename(), attachment.file.url)

        if 'slug' in self.kwargs:
            page = models.get_page_model().objects.get(site=self.request.page_site, slug=self.kwargs['slug'])
            if page:
                for file in page.page_file.all():
                    if file.filename().lower().endswith(('jpg', 'jpeg', 'gif', 'tiff', 'png')):
                        if (wrote_first):
                            response += ","
                        else:
                            wrote_first = True
                        response += "{title: '%s', value: '%s'}" % (file.filename(), file.file.url)

        response += "]"
        return HttpResponse(response)


class Upload_File(View):
    # sizeLimit = 1024000

    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        if not request.user.has_perm('page.add_file'):
            return HttpResponse('Sorry, You do not have rights!')

        return super(Upload_File, self).dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        files = []

        f = request.FILES['files[]']
        wrapped_f = UploadedFile(f)
        filename = wrapped_f.name
#        filesize = wrapped_f.file.size

        uuidfile = models.UUID_File(
            file=File(f, filename),
            uuid=self.request.POST['uuid']
        )
        uuidfile.save()

        files.append({
            "name": filename,
            "url": uuidfile.file.url,
            "pk": uuidfile.pk
        })

        response = {
            'files': files
        }
        return HttpResponse(json.dumps(response))



from django.core.urlresolvers import reverse
from django.forms import ModelForm
import django.forms
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit,Field,Layout,ButtonHolder,Fieldset,HTML,Div,Hidden

from django.utils.encoding import smart_text
from django.core.exceptions import ValidationError
import re
from . import models

BUTTONS = Layout (
              ButtonHolder(
                Submit('submit', 'Save',css_id='save_button'),
                Submit('submit', 'Save and Close',css_id='save_close_button'),
                HTML('&#160;<a href="/{{object.slug}}" class="btn btn-danger" " >Close</a>')
                )
          )

slug_re = re.compile(r'^[-/\w]+$')
class Site(ModelForm):
    class Meta:
        model = models.get_site_model()
        exclude = ()
        
    def __init__(self, *args, **kwargs):
        self.helper = FormHelper()
        self.helper.render_unmentioned_fields=True
        self.helper.layout = Layout(
            Fieldset(
                     'Edit {{ object }}',
                     'title',
                     'domain_name',
                     'default_domain',
                     'meta_data',
                     'footer',
                     ),
           BUTTONS
        )
        super(Site, self).__init__(*args,**kwargs)

class Page_Abstract(ModelForm):
    file = django.forms.FileField(required=False)
    
    class Meta:
        model = models.get_page_model()
        fields = ['slug','title','members_only','content','html_title','sitemap']
        abstract=True
        
    def __init__(self, *args, **kwargs):
        self.site = kwargs.pop('site')
        super(Page_Abstract, self).__init__(*args,**kwargs)

        self.helper = FormHelper(self)
#        self.helper.form_class = 'form-horizontal'
#        self.helper.label_class = 'col-lg-2'
#        self.helper.field_class = 'col-lg-8'
        self.helper.form_id = 'page_edit_form'

        self.page_options =Layout(
            Field(Hidden('uuid', '{{ uuid }}')),
            Fieldset(
                     'Edit {{ request.GET.slug}}',
                     'slug',
                     'title',
                     'html_title',
                     'members_only',
                     'sitemap'
                     )
                                 
                )
        self.file_layout = Layout (
                                   
                        Fieldset(
                         'Files',
                          'file',
                          HTML("{% include \"page/file_upload.html\" %}"),
                          
                          ),
                       )
        

        self.helper.layout = Layout(
                 self.page_options,
                 'content',
                 self.file_layout,
                HTML("<br />"),
               BUTTONS
        )
        
    def clean_slug(self):
        slug = self.cleaned_data['slug'].lower()

        if not slug_re.search(smart_text(slug)):
            raise ValidationError("Enter a valid 'slug' consisting of letters, numbers, underscores, slashes or hyphens.")

        slug = slug.rstrip('/')
        if slug != self.instance.slug and models.get_page_model().objects.filter(site=self.site,slug=slug).exists():
            raise ValidationError("Sorry that name is already taken")
        
        return slug

class Page(Page_Abstract):
    pass

class Page_Show_In_Menu(Page_Abstract):
    class Meta:
        model = models.get_page_model()
        fields = ['slug','title','members_only','priority','content','show_in_menu','html_title','sitemap']
        
    def __init__(self, *args, **kwargs):
        super(Page_Show_In_Menu, self).__init__(*args,**kwargs)
        self.helper.layout = Layout(
                 self.page_options,
                 'show_in_menu',
                 'priority',
                 'content',
                 self.file_layout,
                 HTML("<br />"),
                 BUTTONS 
        )



# -*- coding: utf-8 -*-


from django.db import models, migrations
import datetime
import page.models


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='File',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('file', models.FileField(max_length=255, upload_to=page.models.upload_file_to)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Page',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.CharField(max_length=100, db_index=True)),
                ('title', models.CharField(max_length=100, verbose_name=b'Menu Title')),
                ('show_in_menu', models.BooleanField(default=False)),
                ('priority', models.IntegerField(default=0)),
                ('html_title', models.CharField(max_length=100, null=True, blank=True)),
                ('content', models.TextField()),
                ('members_only', models.BooleanField(default=False)),
                ('lastmod', models.DateTimeField(default=datetime.datetime.now)),
                ('sitemap', models.BooleanField(default=True, verbose_name=b'Share a link to this page with search engines')),
            ],
            options={
                'abstract': False,
                'permissions': (('change_show_in_menu', 'change_show_in_menu'),),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Page_Editor',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('page', models.ForeignKey(to='page.Page')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Site',
            fields=[
                ('title', models.CharField(max_length=200)),
                ('domain_name', models.CharField(max_length=300, serialize=False, primary_key=True)),
                ('default_domain', models.BooleanField(default=False)),
                ('meta_data', models.TextField(null=True, blank=True)),
                ('footer', models.TextField(null=True, blank=True)),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='UUID_File',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('file', models.FileField(max_length=255, upload_to=page.models.upload_uuid_file_to)),
                ('uuid', models.CharField(max_length=100)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='page',
            name='site',
            field=models.ForeignKey(to='page.Site'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='file',
            name='page',
            field=models.ForeignKey(related_name=b'page_file', to='page.Page'),
            preserve_default=True,
        ),
        migrations.CreateModel(
            name='Page_User',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('auth.user',),
        ),
        migrations.AddField(
            model_name='page_editor',
            name='editor',
            field=models.ForeignKey(to='page.Page_User'),
            preserve_default=True,
        ),
    ]



from django import template
from django.core.urlresolvers import reverse 

from django.template.defaultfilters import stringfilter

register = template.Library()

@register.filter
def active_page(page,request):
    """Returns True if active page
    """
    if request.path == '/' and page.slug == 'default':
        return True
    elif request.path == ('/' + page.slug):
        return True
    elif request.path == reverse("page_edit",args=[page.slug]): 
        return True
    
    return False




from . import models

def site_context(request):
    context = {}
    context['template'] = 'desktop.html'
    
    context['menus'] = models.get_page_model().objects.filter(show_in_menu=True).order_by('-priority','title')
    if request.META['REMOTE_ADDR'] == '127.0.0.1':
        context['DEV_SERVER'] =True
    else:
        context['DEV_SERVER'] = False 

    if hasattr(request,'page'):
        context['page'] = request.page

    if hasattr(request,'page_site'):
        context['page_site'] = request.page_site

    return context